#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use strict "vars";

#===============================================================================
# This script will read metabolite extracted data file, checking for each 
# metabolite name for existance. If all names are existed in "compound" or
# "cpd_derivative" then insert data into the table "sample", "sample_extract",
# "metabolite_measurement", "metabolite_processsed2", "metabolite_info" and 
# "metabolite_derivative_exp_set". If any names are not in "compound" or 
# "cpd_derivativi" return the names and exit.
#
# The format for data file is:
# Name\tRet_time\tPeak_ion\tValue[\tSOFTWARE::Library\tSOFTWARE::Annotation\tSOFTWARE::Quantifire]\n
# name_1\t_1\pi_1\v1[\tsl1\tsa1\tsq11[\tsq_12\t..\tsq_1K]]\n
# name_2\t_2\pi_2\v2[\tsl2\tsa2\tsq21[\tsq_22\t..\tsq_2K]]\n
# ...
# name_M\t_M\pi_M\vM[\tslM\tsaM\tsqM1[\tsq_M2\t..\tsq_MK]]\n
#
# where columns starts with SOFTWARE:: are optional information. SOFTWARE is
# the acture software name in the DOME;
# vj, j = 1, ...M are the extracted value for metabolites.
# $sq_jk, k = 1 .. K are quantifiers for the software. 
# There can be one or more softwares. For each software it can have one or more
# quantifiers.
# 
# arguments passed in to this script are:
# $proc_data_file -- the full name of the input processed data file;
# $cdf_file -- the full name of the input raw data file;
# $outputfile -- The full name of the output file name for error messages 
# $file_home -- full name where the files will be copied
# $technique_id -- for GC/MS, LC/MS, CE/MS or other technique;
# $srid -- sampling_replicate_id
# $tech_repl -- technical replicate
# $person_id -- who did the measurement
# $extract_id -- id for extraction
# $exp_prot_id -- for experimental protocol_id
# $stat_ptid -- statistical protocol_type_id
# $tissue_type_label_id -- tissue_type lable
# $reference -- sample reference
# $norm -- 'Y' or 'N'. This data set is normalized or not.
#
# Author: X. Jing Li
# Date: July 11, 2005
#  9/21/2006: add code for software informations. Added new tables "library",
#    "metabolite_info_detail" and "qualifier" for the extra columns of info.
# 10/5/2006: change the script so that new parameter $norm will be taken. 
#    $norm = 'Y' means that the values have been normailized. So even if there 
#    is a reference cpd the value of cpds will NOT be divided by the value of 
#    reference.
# 11/03/2006: Add code to check for repeated names. If there are repeated naems
#    in data file. reject the file and give the error msg about the names and 
#    line number.
#==============================================================================

my ($proc_data_file, $cdf_file, $outputfile, $file_home, $technique_id, $srid,
    $tech_repl, $person_id, $extract_id, $exp_prot_id, $$stat_ptid, 
    $tissue_type_label_id, $reference, $norm,
    $database, $username, $password, $bnet_DB, $specieDB, $dbh, $sth, $rc, $sql,
    $kw_table, $kwid, $pt_name, $desc, $replicate, @temps, $proc_file_name, 
    $cdf_file_name, $proc_file_search_name, $cdf_file_search_name, $mmid,
    $proc_file_id, $cdf_file_id, @contents, $line, $i, $ref_value,
    $r_names, %names, %unique_names, %repeated_names, $r_ret_times, %ret_times, $r_peak_ions, %peak_ions, $line_num,
    $r_values, %values, $name, $all_names, @fields, $ref_key, $exp_set_id,
    $r_name_cpd_ids, %name_cpd_ids, $r_name_cpd_der_ids, %name_cpd_der_ids,
    $r_missing_names, @missing_names, @name_keys, $key, $search_name, $cpd_id,
    $cpd_der_id, $orig_name, $temp_name, $index, $search_names, 
    $no_cpd_id_names, $proc_file_location_link_id, $file_type, $proc_file_type_id, 
    $cdf_file_location_link_id, $cdf_file_type_id, 
    $sample_id, $sample_extract_id, $ret, $peak, $intensity,
    $norm_int, $ref_peak, $miid, $mdeid, $mp2id, %sum_intensities,
    $header, %headers, $key, $software, %softwares, $software_id, $qualifier, $msg, $exist_software_info, $ref_info, @info_array, %info, $mid_kwid, $mid_id, $library_name, $library_id, $annot, $qualifier_value, %miid_sids, $miid_key, $ok
    );

($proc_data_file, $cdf_file, $outputfile, $file_home, $technique_id, $srid, $tech_repl, $person_id, $extract_id, $exp_prot_id, $stat_ptid, $tissue_type_label_id, $reference, $norm) = @ARGV;
#print "proc_data_file = $proc_data_file, cdf_file = $cdf_file, outputfile = $outputfile, file_home = $file_home, technique_id = $technique_id, srid = $srid, tech_repl = $tech_repl, person_id = $person_id, extract_id = $extract_id, exp_prot_id = $exp_prot_id, stat_ptid = $stat_ptid, tissue_type_label_id = $tissue_type_label_id, reference = $reference, norm = $norm\n";
#print "@ARGV\n";

#== global parameters:
$kw_table = "COMPOUND";

#open (TEMP, ">./tmp-plots/readDataFromMetabolite_2.log");
#== 1. connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#== 2. get the kw_table_id for COMPOUND:
$sql = "select kw_table_id from ref_kw_table where upper(kw_table) = \'$kw_table\'";
$sth = $dbh->prepare($sql) or die "Can not prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can not execute: $DBI::errstr.";
$kwid = $sth->fetchrow();

#== get protocol_type_id. if there is no reference then protocol_type_name is 
#== "measured intensity". otherwise it is "Normalized by internal reference":
#if ($reference eq "") {
#    $pt_name = "Measured Intensity";
#    $desc = "Raw Intensity";
#} else {
#    $pt_name = "Normalized by Internal Reference";
#    $desc = "Raw Intensity divided by reference intensity";
#}
#$ptid = getProtocolTypeId($dbh, $pt_name, $extract_id);

#if ($ptid eq "") {
#    $ptid = createProtocolTypeId($dbh, $pt_name, $desc, $extract_id);
#}

#== 3. check if the files have been uploaded before:
@temps = split /\//, $proc_data_file;
$proc_file_name = pop @temps;
@temps = split /\//, $cdf_file;
$cdf_file_name = pop @temps;
$proc_file_search_name = $proc_file_name;
$proc_file_search_name =~ s/\'/\'\'/g;
$proc_file_search_name =~ s/\\//g;
$cdf_file_search_name = $cdf_file_name;
$cdf_file_search_name =~ s/\'/\'\'/g;
$sql = "select metabolite_measurement_id from metabolite_measurement where proc_file_location_link_id in (select file_location_link_id from file_location_link where file_id in (select file_id from \"file\" where file_name = \'$proc_file_search_name\') and file_location_id in (select file_location_id from file_location where \"location\" = \'$file_home\')) or raw_file_location_link_id in (select file_location_link_id from file_location_link where file_id in (select file_id from \"file\" where file_name = \'$cdf_file_search_name\') and file_location_id in (select file_location_id from file_location where \"location\" = \'$file_home\'))";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$mmid = $sth->fetchrow();
if ($mmid ne "") {
    $sql = "select file_id from \"file\" where file_name = \'$proc_file_name\'";
    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $proc_file_id = $sth->fetchrow();
    $sql = "select file_id from \"file\" where file_name = \'$cdf_file_name\'";
    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $cdf_file_id = $sth->fetchrow();
    if ($cdf_file_id ne "" && $cdf_file_name ne "") {
        print "ERROR\tFiles $proc_file_name and $cdf_file_name have been uploaded before\n";
        $dbh->rollback;
        $sth->finish;
        $dbh->disconnect;
        exit 1;
    } elsif ( $cdf_file_id ne "") {
        print "ERROR\tFile $cdf_file_name has been uploaded before\n";
        $dbh->rollback;
        $sth->finish;
        $dbh->disconnect;
        exit 1;
    } elsif ( $proc_file_id ne "") {
        print "ERROR\tFile $proc_file_name has been uploaded before\n";
        $dbh->rollback;
        $sth->finish;
        $dbh->disconnect;
        exit 1;
    }
}

#== 4. check the technical_replicate to see if it has been uploaded:
$sql = "select MM.metabolite_measurement_id, SE.technical_replicate from metabolite_measurement MM, sample_extract SE where MM.technique_id = $technique_id and MM.protocol_id = $exp_prot_id and MM.sample_extract_id = SE.sample_extract_id and SE.technical_replicate = $tech_repl and SE.sample_id in (select sample_id from sample where sampling_replicate_id = $srid and label_id= $tissue_type_label_id)";
#print "$sql\n";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($mmid, $replicate) = $sth->fetchrow()) {
    if ($mmid ne "") {
        print getErrorDetail($dbh, $technique_id, $srid, $replicate, $exp_prot_id, $tissue_type_label_id, $reference);
        $dbh->rollback;
        $sth->finish;
        $dbh->disconnect;
        exit 1;
    }
}

#== if get here then the sample is a new one. start to parse the proc_file:
#== 5. get all software names:
$sql = "select software_id, software_name from software";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($software_id, $software) = $sth->fetchrow()) {
    $software = uc $software;
    $softwares{$software} = $software_id;
}

#== 6. read the input and output files:
open (IN, "<$proc_data_file") || die "Can not open the input file $proc_data_file.$!\n";
@contents = <IN>;
close IN;
$line = shift @contents;
$line =~ s/[\n\r]|\"//g;
chomp $line;
@fields = split /\t/, $line;
$index = 0;
$exist_software_info = 0;
foreach $header (@fields) {
    if ($header =~ /^name/i) {
        $headers{"NAME"} = $index;
    } elsif ($header =~ /^ret_time/i) {
        $headers{"RET_TIME"} = $index;;
    } elsif ($header =~ /^peak_ion/i) {
        $headers{"PEAK_ION"} = $index;
    } elsif ($header =~ /^value/i) {
        $headers{"VALUE"} = $index;
    } elsif ($header =~ /\:{2}/) {
        $exist_software_info = 1;
        #-- this is a qualifier from a software:
        ($software, $qualifier) = split /\:\:/, $header;
        $software = uc ($software);
        if (!exists $softwares{$software}) {
            $msg = "The software \"$software\" is not in DOME. You need to add it to DOME before you can proceed by going to main page \"Database management\" -> \"Add MetaData\" -> \"Add software\". If you do not the privilage, please ask the manager to do so.\n";
            print $msg;
            $dbh->rollback;
            $sth->finish;
            $dbh->disconnect;
            exit 1;
        } else {
            $software_id = $softwares{$software};
            if ($qualifier =~ /library/i) {
                $key = $software_id . "::LIBRARY";
                $headers{$key} = $index;
            } else {
                $key = $software_id . "::$qualifier";
                $headers{$key} = $index;
            }
        }
    } else {
        print "Unknown file header \"$header\". Please look at the \"Help\" page for the file format.\n";
        $dbh->rollback;
        $sth->finish;
        $dbh->disconnect;
        exit 1;
    }
    $index++;
}

#== if there are columns related to software. get the kw_table_id for metabolite_info_detail:
if ($exist_software_info == 1) {
    $sql = "select kw_table_id from ref_kw_table where kw_table = 'METABOLITE_INFO_DETAIL'";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $mid_kwid = $sth->fetchrow();
}

#= 7. parse the lines and save the info in hashes:
$r_names = {};
%names = %$r_names;
$r_ret_times = {};
%ret_times = %$r_ret_times;
$r_peak_ions = {};
%peak_ions = %$r_peak_ions;
$r_values = {};
%values = %$r_values;
$all_names = "\'" . $reference . "\'";
for ($i = 0; $i<@contents; $i++) {
    $line = $contents[$i];
    $line =~ s/[\n\r]|\"//g;
    chomp $line;
    if ($line !~ /\d|\w/) {
        next;
    }
    @fields = split /\t/, $line;
    $name = $fields[$headers{"NAME"}];
    $names{$i} = $name;
    $search_name = uc(getSearchName($name));
    #print "line = $i, name = $name, search_name = $search_name\n";
    if (!exists $unique_names{$search_name}) {
        $unique_names{$search_name} = $i;
    } else {
        $repeated_names{$i} = $search_name;
    }
    $ret_times{$i} = $fields[$headers{"RET_TIME"}];
    $peak_ions{$i} = $fields[$headers{"PEAK_ION"}];
    $values{$i} = $fields[$headers{"VALUE"}];
    if ($reference ne "" && uc($name) eq uc($reference) ) {
        $ref_key = $i;
        $ref_value = $values{$i};
    } else {
        $name =~ s/\'/\'\'/g;
        $all_names .= ", \'$name\'";
    }
    if ($exist_software_info == 1){
        $info{$i} = createArrayRef(\@fields);
    }
}

#== check if htere are repeated cpd names:
if (keys %repeated_names > 0) {
    $msg = "Duplicated metabolite names:\n";
    foreach $i (sort {$a <=> $b} keys %repeated_names) {
        $line_num = $i+2;
        $msg .= "In line $line_num: \"" . $names{$i} . "\" is the same as in line " . ($unique_names{$repeated_names{$i}} + 2) . ".\n";
    }
    $msg .= "You must give each metabolit a unique name. If it is a derivative of another metabolite, please indicate it as a derivative of that metabolite.\n";
    print $msg;
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    exit 1;
}
    


#== 8. check the reference is in the name list if reference is not null. If Yes 
#== print out the error and quit. Otherwise script will go to next step:
if ($reference ne "" && $ref_key eq "") {
    print "ERROR\tReference $reference is not in the data file\n";
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    exit 1;
}

#== 9. get the exp_set_id:
$sql = "select exp_set_id from experiment where experiment_id in "
    . "(select experiment_id from sampling_point where sampling_point_id in "
    . "(select sampling_point_id from sampling_replicate where "
    . "sampling_replicate_id = $srid))";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$exp_set_id = $sth->fetchrow();

#== 10. check all names in "compound" and/or "cpd_derivative":
$r_name_cpd_ids = {};
%name_cpd_ids = $r_name_cpd_ids;
$r_name_cpd_der_ids = {};
%name_cpd_der_ids = $r_name_cpd_der_ids;
$r_missing_names = ();
@missing_names = @$r_missing_names;
@name_keys = sort {$a <=> $b} keys %names;
$all_names = uc $all_names;
#= 10.1 check all names in compound_derivative:
$sql = "select cpd_derivative_id, cpd_id, name from cpd_derivative "
    . "where upper(name) in ($all_names)";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($cpd_der_id, $cpd_id, $name) = $sth->fetchrow ) {
    if ($cpd_der_id ne "") {
        foreach $key (@name_keys) {
            $orig_name = uc $names{$key};
            $orig_name =~ s/\"|\s+|\-+//g;
            $temp_name = uc $name;
            $temp_name =~ s/\"|\s+|\-+//g;
            if ($orig_name eq $temp_name) {
                $name_cpd_der_ids{$key} = $cpd_der_id;
                $name_cpd_ids{$key} = $cpd_id;
                last;
            }
        }
    }
}

#= 10.2 check all names in compound:
$search_names = $all_names;
$search_names =~ s/\"|\s+|\-+//g;
$sql = "select record_id, aname from allnames where kw_table_id = $kwid "
    . "and upper(search_name) in ($search_names)";
#print $sql . "\n";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($cpd_id, $name) = $sth->fetchrow ) {
    if ($cpd_id ne "") {
        foreach $key (@name_keys) {
            $orig_name = uc $names{$key};
            $orig_name =~ s/\"|\s+|\-+//g;
            $temp_name = uc $name;
            $temp_name =~ s/\"|\s+|\-+//g;
            if ($orig_name eq $temp_name) {
                #print "key = $key, cpd_id = $cpd_id, orig_name = $orig_name, temp_name = $temp_name\n";
                $name_cpd_ids{$key} = $cpd_id;
                last;
            }
        }
    }
}

#== 11. find out the missing names:
foreach $key (@name_keys) {
    #print $key, "\t", $names{$key}, "\t", $name_cpd_ids{$key}, "\n";
    if (!exists $name_cpd_ids{$key}) {
        push @missing_names, $names{$key};
    }
}
#print "@missing_names\n";

#= if @missing_names is not empty, it means there are names are not in database.
#= return the names which do not have a cpd_id to user. Otherwise it means all 
#= names are OK, then start insertion to database.
if (@missing_names > 0) {
    $no_cpd_id_names = "";
    foreach $name (@missing_names) {
        $no_cpd_id_names .= "NOTFOUND\t$name\n";
    }
    open (OUT, ">$outputfile") or warn "Can not open output data file $outputfile\n";
    print OUT $no_cpd_id_names;
    close OUT;
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    exit 99;
}  

#== 12. all names are in database. start to insert new records into tables.
#= 12.1 copy the files into file_location, create file_id, file_location_link_id
#= for both cdf_file and proc_file:
$proc_file_type_id = getFileTypeId($dbh, $proc_file_name);
if($proc_file_type_id eq "") {
    $proc_file_type_id = createFileTypeId($dbh, $proc_file_name);
}
$proc_file_location_link_id = createFileIds($dbh, $file_home, $proc_data_file, $proc_file_type_id, $person_id);
if ($cdf_file_name ne "") {
    $cdf_file_type_id = getFileTypeId($dbh, $cdf_file_name);
    if($cdf_file_type_id eq '') {
        $cdf_file_type_id = createFileTypeId($dbh, $cdf_file_name);
    }
    $cdf_file_location_link_id = createFileIds($dbh, $file_home, $cdf_file, $cdf_file_type_id, $person_id);
}

#= 12.2 a new record in sample and sample_extract:
$sample_id = getSampleIdFromSampleExtract($dbh, $srid, $extract_id, $tissue_type_label_id);
if ($sample_id eq "") {
    $sample_id = createSampleId($dbh, $srid, $tissue_type_label_id);
    if ($sample_id eq "") {
        $dbh->rollback;
        $sth->finish;
        $dbh->disconnect;
        print "Failed to create \"sample_id\"\n";
        exit 1;
    }
}

#= 12.3 a new record in sample_extract:
$sample_extract_id = getSampleExtractId($dbh, $sample_id, $extract_id, $tech_repl);
if($sample_extract_id eq "") {
    $sample_extract_id = createSampleExtractId($dbh, $sample_id, $extract_id, $tech_repl);
    if ($sample_extract_id eq "") {
        $dbh->rollback;
        $sth->finish;
        $dbh->disconnect;
        print "Failed to create \"sample_extract_id\"\n";
        exit 1;
    }
}

#= 12.4 new metabolite_measurement_ids:
$mmid = createMetaboliteMeasurementIdWithInputFilelocationLinkIds($dbh, $sample_extract_id, $technique_id, $exp_prot_id, $tech_repl, $person_id, $proc_file_location_link_id, $cdf_file_location_link_id);
if ($mmid eq "") {
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    print "Failed to create \"metabolite_measurement_id\"\n";
    exit 1;
}

#= 12.5 metabolite_processed2 and metabolite_info_detail, library, quantifier
#= for all replicates
foreach $key (@name_keys) {
    $intensity = $values{$key};
    if ($intensity eq "") {
        next;
    }
    $peak = $peak_ions{$key};
    $ret = $ret_times{$key};
    $cpd_id = $name_cpd_ids{$key};
    $cpd_der_id = $name_cpd_der_ids{$key};
    if ($key eq $ref_key && $ref_key ne "") {
        $ref_peak = "Y";
    } else {
        $ref_peak = "N";
    }
    $miid = insertMetaboliteInfo($dbh, $mmid, $cpd_der_id, $cpd_id, $peak, $intensity, $ref_peak);
    if ($miid eq "") {
        $dbh->rollback;
        $sth->finish;
        $dbh->disconnect;
        print "Failed to insert data into \"metabolite_info\"\n";
        exit 1;
    }

    #= save the intensity in hash for metabolite_processed2 table.
    $sum_intensities{$cpd_id} += $intensity;

    $mdeid = insertMetaboliteDerivExpSet($dbh, $miid, $exp_set_id, $ret);
    if ($mdeid eq "") {
        $dbh->rollback;
        $sth->finish;
        $dbh->disconnect;
        print "Failed to insert data into \"metabolite_derivative_exp_set\"\n";
        exit 1;
    }

    #= get the additional info to database:
    if ($exist_software_info == 1) {
        $ref_info = $info{$key};
        @info_array = @$ref_info;

        %miid_sids = {};
        foreach $header (keys %headers) {
            $ok = "";
            if ($header =~ /\:\:/){
                ($software_id, $qualifier) = split /\:\:/, $header;
                $miid_key = $miid . "_" . $software_id;
                if (!exists $miid_sids{$miid_key}) {
                    $mid_id = createMetaboliteInfoDetailId($dbh, $miid, $software_id);
                    $miid_sids{$miid_key} = $mid_id;
                } else {
                    $mid_id = $miid_sids{$miid_key};
                }

                if ($qualifier eq "LIBRARY"){
                    $library_name = $info_array[$headers{$header}];
                    if ($library_name ne "") {
                        $library_id = getLibraryId($dbh, $library_name);
                        if ($library_id eq "") {
                            $library_id = createLibraryId($dbh, $library_name);
                        }
                        $ok = updateMetaboliteInfoDetailTable($dbh, $mid_id, $software_id, "LIBRARY_ID", $library_id);
                    }
                } elsif ($qualifier =~ /ANNOTATION/i){
                    $annot = $info_array[$headers{$header}];
                    if ($annot ne "") {
                        $ok = updateMetaboliteInfoDetailTable($dbh, $mid_id, $software_id, "ANNOTATION", $annot);
                    }
                } else {
                    $qualifier_value = $info_array[$headers{$header}];
                    if ($qualifier_value =~ /\w/) {
                        $ok = createQualifierId($dbh, $mid_kwid, $mid_id, $qualifier, $qualifier_value);
                    }
                }
            }
        }
    }
}

#== 12.6. insert processed data into metabolite_processed2:
foreach $cpd_id (keys %sum_intensities) {
    $intensity = $sum_intensities{$cpd_id};
    if ($norm eq "Y") {
        if ($reference ne "") {
            if ($ref_value != 0) {
                $norm_int = $intensity / $ref_value;
            } else {
                $norm_int = 'null';
            }
            $mp2id = insertMetaboliteProcessed2($dbh, $cpd_id, $intensity, $norm_int, $mmid, $stat_ptid);
            if ($mp2id eq "") {
                $dbh->rollback;
                $sth->finish;
                $dbh->disconnect;
                print "Failed to insert normalized metabolite date.\n";
                exit 1;
            }
        } elsif ($reference eq "") {
            $mp2id = insertMetaboliteProcessed2($dbh, $cpd_id, $intensity, $intensity, $mmid, $stat_ptid);
            if ($mp2id eq "") {
                $dbh->rollback;
                $sth->finish;
                $dbh->disconnect;
                print "Failed to insert data into \"metabolite_processed2\"\n";
                exit 1;
            }
        }
    } else {
        $mp2id = insertMetaboliteProcessed2($dbh, $cpd_id, $intensity, $intensity, $mmid, $stat_ptid);
        if ($mp2id eq "") {
            $dbh->rollback;
            $sth->finish;
            $dbh->disconnect;
            print "Failed to insert data into \"metabolite_processed2\"\n";
            exit 1;
        }
    }
}
close TEMP;
print "Inserting metabolite data from $proc_file_name to DOME is successful.\n";
$dbh->commit;
#$dbh->rollback;
$sth->finish;
$dbh->disconnect;

exit 0;

###########################################################################
# subroutine:
#  createFileIds($dbh, $file_home, $file_name, $file_type_id, person_id)
#
# parameters in: $dbh --- the database handle,
#                $file_namename --- the file name
# parameter out: $_id --- the id.
#
# X. Jing Li, 4/12/2005
###########################################################################
sub createFileIds {
    my ($dbh, $file_home, $file_name, $file_type_id, $person_id) = @_;
    my ($sql, $sth, $rc, $flid, $fid, $fllid, @temp, $file_suffix, $fname,
        $file_type, $new_fname, $cmd);

    #== get file_location_id for file_home:
    $sql = "select file_location_id from file_location where location = \'$file_home\'";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $flid = $sth->fetchrow();
    if ($flid eq "") {
        $flid = createFileLocationId($dbh, $file_home);
    }

    #== create file_id:
    @temp = split "/", $file_name;
    $fname = pop @temp;
    $fname =~ s/\'/\'\'/g;

    $sql = "select nextval('file_id_seq')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $fid = $sth->fetchrow;

    $sql = "insert into \"file\" (file_id, file_name, file_type_id, time_uploaded, person_id) values ($fid, \'$fname\', $file_type_id, (SELECT abstime(timestamp 'now')), $person_id)";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    #== create file_location_link_id:
    $sql = "select nextval('file_location_link_id_seq')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $fllid = $sth->fetchrow;

    $sql = "insert into file_location_link (file_location_link_id, file_id, file_location_id) values ($fllid, $fid, $flid)";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    if ($fid ne "" && $flid ne "" && $fllid ne "") {
        @temp = split "/", $file_name;
        $fname = pop @temp;
        $fname =~ s/\'/\\\'/g;
        $new_fname = $file_home . $fname;
        $cmd = "cp \"$file_name\" \"$new_fname\"\n";
        system $cmd;
        unlink ($file_name) or warn "Can not delete file $file_name from holding diretory. $!\n";
    }
    $sth->finish;
    return $fllid;
}

###########################################################################
# subroutine: getFileTypeId($dbh, $file_name)
#
# parameters in: $dbh --- the database handle,
#                $file_name --- the file home,
# parameter out: $ftid --- the new file_type_id;
#
# X. Jing Li, 7/20/2005
###########################################################################
sub getFileTypeId {
    my ($dbh, $file_name) = @_;
    my ($sql, $sth, $rc, @temp, $file_type_id);

    @temp = split /\./, $file_name;
    $file_type = "." . pop @temp;
    $sql = "select file_type_id from file_type where upper(\"file_suffix\") = \'" . (uc $file_type) . "\'";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $file_type_id = $sth->fetchrow();

    $sth->finish;
    return $file_type_id;
}

###########################################################################
# subroutine: createFileTypeId($dbh, $file_type)
#
# parameters in: $dbh --- the database handle,
#                $file_home --- the file home,
# parameter out: $ftid --- the new file_type_id;
#
# X. Jing Li, 4/29/2005
###########################################################################
sub createFileTypeId {
    my ($dbh, $file_name) = @_;
    my ($sql, $sth, $rc, @temp, $ftid);
 
    @temp = split /\./, $file_name;
    $file_type = "." . pop @temp;

    $sql = "select nextval('file_type_id_seq')"; 
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $ftid = $sth->fetchrow();

    $sql = "insert into \"file_type\" (file_type_id, file_suffix) values ($ftid, \'$file_type\')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $sth->finish;
    return $ftid;
}
 
###########################################################################
# subroutine: createFileLocationId($dbh, $file_home)
#
# parameters in: $dbh --- the database handle,
#                $file_home --- the file home
# parameter out: $flid --- the new file_location_id.
#
# X. Jing Li, 4/29/2005
###########################################################################
sub createFileLocationId {
    my ($dbh, $file_home) = @_;
    my ($sql, $sth, $rc, $flid);

    $sql = "select nextval('file_location_id_seq')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $flid = $sth->fetchrow();

    $sql = "insert into file_location (file_location_id, location) values ($flid, \'$file_home\')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $sth->finish;
    return $flid;
}

###########################################################################
# subroutine: getErrorDetail
#
# parameters in: $dbh -- the database handle,
#                $technique_id --
# parameter out: $_id --- the id.
#
# X. Jing Li, 4/12/2005
###########################################################################
sub getErrorDetail {
    my ($dbh, $technique_id, $srid, $tech_repl, $protocol_id, $tissue_type_label_id, $reference) = @_;
    my ($sql, $sth, $rc, $mmid, $rfllid, $pfllid, $exp_name, $exp_set_name, 
        $lname, $raw_file_name, $upload_time, $person_name, $error_msg);

    #== get the detail:
    $sql = "select MM.metabolite_measurement_id, MM.raw_file_location_link_id, MM.proc_file_location_link_id, E.exp_name, ES.exp_set_name, L.\"name\" from metabolite_measurement MM, experiment E, experiment_set ES, \"sample\" S, sample_extract SE, label L, sampling_point SP, sampling_replicate SR where MM.protocol_id = $protocol_id and MM.sample_extract_id = SE.sample_extract_id and SE.extract_id = $extract_id and SE.sample_id = S.sample_id and SE.technical_replicate = $tech_repl and S.sampling_replicate_id = SR.sampling_replicate_id and SR.sampling_replicate_id = $srid and SR.sampling_point_id = SP.sampling_point_id and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($mmid, $rfllid, $pfllid, $exp_name, $exp_set_name, $lname) = $sth->fetchrow();
    
    if ($rfllid ne "") {
        $sql = "select F.file_name, F.time_uploaded, P.person_name from \"file\" F, person P where F.file_id = (select file_id from file_location_link where file_location_link_id = $rfllid) and P.person_id = F.person_id";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
        ($raw_file_name, $upload_time, $person_name) = $sth->fetchrow();
    }
    if ($pfllid ne "") {
        $sql = "select F.file_name, F.time_uploaded, P.person_name from \"file\" F, person P where F.file_id = (select file_id from file_location_link where file_location_link_id = $pfllid) and P.person_id = F.person_id";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
        ($proc_file_name, $upload_time, $person_name) = $sth->fetchrow();
    }

    $error_msg = "ERROR: This set of parameters have been used.\nThe experiment_set \"$exp_set_name\", experiment \"$exp_name\" at time \"$lname\" had been uploaded with file \"$proc_file_name\" by \"$person_name\" on \"$upload_time\"\n";

    $sth->finish;
    return $error_msg;
}

###########################################################################
# subroutine: getSampleIdFromSampleExtract($dbh, $srid):
# Get the sample_id from database.
#
# parameters in: $dbh --- the database handle;
#                $srid --- sampling_replicate_id
# parameter out: $sample_id --- the sample_id associated with the 
#                    sampling_replicate_id..
# X. Jing Li. 8/31/2004
###########################################################################
sub getSampleIdFromSampleExtract {
    my ($dbh, $srid, $extract_id, $tissie_type_label_id) = @_;
    my ($sql, $sth, $rc, $sample_id);

    $sql = "select sample_id from sample_extract where extract_id = $extract_id and sample_id in (select sample_id from sample where sampling_replicate_id = $srid and label_id = $tissie_type_label_id)";
    $sth = $dbh->prepare($sql) or die "Can't prepare select: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";
    $sample_id = $sth->fetchrow;
    $sth->finish;

    return $sample_id;
}

###########################################################################
# 17b. subroutine: createMetaboliteMeasurementIdWithProcInputFile
# ($dbh, $sample_extract_id, $technique_id, $protocol_id, $replicate, 
# $person_id, $proc_input_file):
#  this subroutine create a metabolite_measurement_id in metabolite_measurement
#  table
# 
# parameters in: $dbh --- database handle;
#                $sample_extract_id --- ;
#                $technique_id ---
#                $protocol_id ---
#                $replicate --- the replicate number;
#                $person_id --- the person who did the metabolite extraction
#                $proc_input_file --- the file name on user's local machine
# parameter out: $metabolite_measurement_id --- the new metabolite_measurement
#                                               _id; 
# X. Jing Li, 11/2/2004.
###########################################################################
sub createMetaboliteMeasurementIdWithInputFilelocationLinkIds {
    my ($dbh, $sample_extract_id, $technique_id, $protocol_id, $replicate, $person_id, $proc_file_location_link_id, $raw_file_location_link_id) = @_;
    my ($sql, $sth, $rc);
    my $metabolite_measurement_id;

    $sth = $dbh->prepare("select nextval('metabolite_measurement_id_seq')") 
        or die "Can not prepare select nextval from dual: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $metabolite_measurement_id = $sth->fetchrow;

    $sql = "insert into metabolite_measurement (metabolite_measurement_id,"
        . "sample_extract_id, technique_id, protocol_id, replicate, person_id, "
        . "proc_file_location_link_id";
    if ($raw_file_location_link_id ne '') {
        $sql .= ", raw_file_location_link_id";
    }
    $sql .= ")  values "
        . "(\'$metabolite_measurement_id\', \'$sample_extract_id\', "
        . "\'$technique_id\', \'$protocol_id\', \'$replicate\', \'$person_id\',"
        . " \'$proc_file_location_link_id\'";
    if ($raw_file_location_link_id ne '') {
        $sql .= ", \'$raw_file_location_link_id\'";
    }
    $sql .= ")";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $metabolite_measurement_id;
}

###########################################################################
# subroutine: insertMetaboliteInfo
#         ($dbh, $mmid, $cpd_der_id, $cpd_id, $peak, $intensity, $ref_peak):
# Insert metabolite intensity nto metabolite_info.
#
# parameters in: $dbh -- the database handle,
#                $mmid -- the metabolite_measurement_id,
#                $cpd_der_id -- the cpd_derivative_id,
#                $cpd_id -- the compound_id,
#                $peak -- the peak_ion,
#                $intensity -- the intensity value,
#                ref_peak -- the intensity of the reference,
# parameter out: $miid -- the metabolite_info_id.
# X. Jing Li. 9/2/2004
###########################################################################
sub insertMetaboliteInfo {
    my ($dbh, $mmid, $cpd_der_id, $cpd_id, $peak, $intensity, $ref_peak) = @_;
    my ($sql, $sth, $rc, $miid);

    $sql = "select nextval('metabolite_info_id_seq')";
    $sth = $dbh->prepare($sql) or die "Can not prepare selectl: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $miid = $sth->fetchrow;

    if ($cpd_der_id eq "") {
        $sql = "insert into metabolite_info (metabolite_info_id, metabolite_measurement_id, cpd_id, peak_ion, intensity, reference_peak) values (\'$miid\', \'$mmid\', \'$cpd_id\', \'$peak\', \'$intensity\', \'$ref_peak\')";
    } else {
        $sql = "insert into metabolite_info (metabolite_info_id, metabolite_measurement_id, cpd_derivative_id, cpd_id, peak_ion, intensity, reference_peak) values (\'$miid\', \'$mmid\', \'$cpd_der_id\', \'$cpd_id\', \'$peak\', \'$intensity\', \'$ref_peak\')";
    }
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $miid;
}

###########################################################################
# subroutine: insertMetaboliteDerivExpSet($dbh, $miid, $exp_set_id, $ret):
# Insert new record into metabolite metabolite_derivative_exp_set..
#
# parameters in: $dbh -- the database handle,
#                $miid -- the metabolite_info_id,
#                $exp_set_id -- the exp_set_id,
#                $ret -- the retention time,
# parameter out: $mdeid -- the metabolite_info_id.
# X. Jing Li. 9/2/2004
###########################################################################
sub insertMetaboliteDerivExpSet {
    my ($dbh, $miid, $exp_set_id, $ret) = @_;
    my ($sql, $sth, $rc, $mdeid);

    $sql = "select nextval('metabolite_deriv_exp_set_id_seq')";
    $sth = $dbh->prepare($sql) or die "Can not prepare: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $mdeid = $sth->fetchrow;

    $sql = "insert into metabolite_derivative_exp_set "
        . "(metabolite_deriv_exp_set_id, metabolite_info_id, exp_set_id, "
        . "retention_time) "
        . "values (\'$mdeid\', \'$miid\', \'$exp_set_id\', \'$ret\')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $mdeid;
}

###########################################################################
# subroutine: 
#  insertMetaboliteProcessed2($dbh, $cpd_id, $intensity, $ref_value, $mmid):
# Insert new record into metabolite_processed2.
#
# parameters in: $dbh -- the database handle,
#                $cpd_id -- the cpd_id,
#                $intensity -- the the intensity of the compound,
#                $norm_int -- the normalized intensity,
#                $mmid -- the metabolite_measurement_id,
# parameter out: $mp2id -- the metabolite_processed2_id.
# X. Jing Li. 9/2/2004
###########################################################################
sub insertMetaboliteProcessed2 {
    my ($dbh, $cpd_id, $intensity, $norm_int, $mmid, $ptid) = @_;
    my ($intensity_l, $norm_int_l, $sql1, $sql2, $sth, $rc, $ratio, $mp2id);

    $sql = "select nextval('metabolite_processed2_id_seq')";
    $sth = $dbh->prepare($sql) or die "Can not prepare: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $mp2id = $sth->fetchrow;

    if ($intensity > 0){
        $intensity_l = log($intensity) / log(2);
    }
    if ($norm_int > 0) {
        $norm_int_l = log($norm_int) / log(2);
    }
    $sql1 = "insert into metabolite_processed2 (metabolite_processed2_id, metabolite_measurement_id, protocol_type_id, cpd_id, intensity, norm_intensity";
    $sql2 = " values ($mp2id, $mmid, $ptid, $cpd_id, $intensity, $norm_int";
    if ($intensity_l ne "") {
        $sql1 .= ", intensity_l";
        $sql2 .= ", $intensity_l";
    }
    if ($norm_int_l ne "") {
        $sql1 .= ", norm_intensity_l";
        $sql2 .= ", $norm_int_l";
    }
    $sql1 .= ")";
    $sql2 .= ")";
    $sql1 .= $sql2;
    #print "$sql1\n";
    $sth = $dbh-> prepare($sql1) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $mp2id;
}

###########################################################################
# subroutine: getProtocolTypeId($dbh, $pt_name, $technique_id)
#
# parameters in: $dbh -- the database handle,
#                $pt_name -- the protocol_type name,
#                $technique_id -- the technique_id
# parameter out: $ptid -- the protocol_type_id.
#
# X. Jing Li, 10/20/2005
###########################################################################
sub getProtocolTypeId {
    my ($dbh, $pt_name, $extract_id) = @_;
    my ($sql, $sth, $rc, $extract_name, $ptid);

    $sql = "select \"name\" from \"extract\" where \"extract_id\" = $extract_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $extract_name = $sth->fetchrow();

    $pt_name .= " -- " . $extract_name;
    $pt_name = uc $pt_name;
    $pt_name =~ s/\'/\'\'/g;
    $sql = "select \"protocol_type_id\" from \"protocol_type\" where upper(\"protocol_type_name\") like \'%$pt_name%\' AND \"technique_id\" = (select \"technique_id\" from \"extract\" where \"extract_id\" = $extract_id) AND \"method\" = \'statistical\'";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $ptid = $sth->fetchrow();
    $sth->finish;
    return $ptid;
}

###########################################################################
# subroutine: createProtocolTypeId($dbh, $pt_name, $desc, $technique_id)
#  This subroutine create a new protocol_type_id and the protocol_id for Mean
# in protocol table.
#
# parameters in: $dbh -- the database handle,
#                $pt_name -- the protocol_type name,
#                $desc -- the protocol_type_desc,
#                $technique_id
# parameter out: $ptid -- the id.
#
# X. Jing Li, 10/20/2005
###########################################################################
sub createProtocolTypeId {
    my ($dbh, $pt_name, $desc, $extract_id) = @_;
    my ($sql, $sth, $rc, $extract_name, $technique_id, $p_name, $ptid);

    $pt_name =~ s/\'/\'\'/g;

    #== get the protocol name for this extract:
    $sql = "select \"name\" from \"extract\" where \"extract_id\" = $extract_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $extract_name = $sth->fetchrow();

    #== get technique_id:
    $sql = "select \"technique_id\" from \"extract\" where \"extract_id\" = $extract_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $technique_id = $sth->fetchrow();
    
    $sth = $dbh->prepare("select nextval('protocol_type_id_seq')") 
         or die "Can not prepare select nextval from dual: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $ptid = $sth->fetchrow;
    $pt_name .= " -- $extract_name";
    $sql = "insert into \"protocol_type\" (\"protocol_type_id\", \"protocol_type_name\", \"protocol_type_desc\", \"method\", \"technique_id\") values ($ptid, \'$pt_name\', \'$desc\', \'statistical\', $technique_id)";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $sql = "insert into \"protocol\" (\"protocol_id\", \"protocol_name\", \"protocol_type_id\", \"protocol_desc\") values (nextval('protocol_id_seq'), 'Mean', $ptid, 'Mean for $pt_name')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $sth->finish;
    return $ptid;
}

###########################################################################
# subroutine: createArrayRef($ref_array)
#
# parameters in: $ref_array -- the reference of an array
# parameter out: $r_array -- the new reference of te array
#
# X. Jing Li, 9/22/2006
###########################################################################
sub createArrayRef {
    my ($ref_array) = @_;
    my @array = @$ref_array;
    return \@array;
}

###########################################################################
# subroutine: getLibraryId($dbh, $library_name)
#
# parameters in: $dbh -- the database handle,
#                $library_name -- 
# parameter out: $library_id -- the new reference of te array
#
# X. Jing Li, 9/25/2006
###########################################################################
sub getLibraryId {
    my ($dbh, $library_name) = @_;
    my ($sql, $sth, $rc, $library_id);

    $library_name =~ s/\'/\'\'/g;
    $sql = "select library_id from \"library\" where upper(library_name) = upper(\'$library_name\')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $library_id = $sth->fetchrow();

    return $library_id;
}

###########################################################################
# subroutine: createLibraryId($dbh, $library_name)
#
# parameters in: $dbh -- the database handle,
#                $library_name -- 
# parameter out: $library_id -- the new reference of te array
#
# X. Jing Li, 9/27/2006
###########################################################################
sub createLibraryId {
    my ($dbh, $library_name) = @_;
    my ($sql, $sth, $rc, $library_id);
    
    $library_name =~ s/\'/\'\'/g;
    $sth = $dbh->prepare("select nextval('library_id_seq')") 
         or die "Can not prepare select nextval: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $library_id = $sth->fetchrow;

    $sql = "insert into \"library\" (library_id, library_name) values ($library_id, \'$library_name\')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    return $library_id;
}

###########################################################################
# subroutine: createMetaboliteInfoDetailId($dbh, $metabolite_info_id)
#
# parameters in: $dbh -- the database handle,
#                $metabolite_info_id --
#                $software_id -- id for the software 
# parameter out: $metabolite_info_detail_id -- the new id
#
# X. Jing Li, 9/25/2006
###########################################################################
sub createMetaboliteInfoDetailId {
    my ($dbh, $metabolite_info_id, $software_id) = @_;
    my ($sql, $sth, $rc, $metabolite_info_detail_id);

    $sql = "select nextval('metabolite_info_detail_id_seq') ";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $metabolite_info_detail_id = $sth->fetchrow();

    $sql = "insert into metabolite_info_detail (metabolite_info_detail_id, metabolite_info_id, software_id) values ($metabolite_info_detail_id, $metabolite_info_id, $software_id)";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $sth->finish;
    return $metabolite_info_detail_id;
}

###########################################################################
# subroutine: updateMetaboliteInfoDetailTable
# ($dbh, $metabolite_info_detail_id, $software_id, $qualifier, $qvalue)
#
# parameters in: $dbh -- the database handle,
#                $metabolite_info_detail_id -- 
#                $software_id --
#                $qualifier -- 
#                $qvalue -- qualifier value
# parameter out: $metabolite_info_detail_id -- the new id
#
# X. Jing Li, 9/25/2006
###########################################################################
sub updateMetaboliteInfoDetailTable {
    my ($dbh, $metabolite_info_detail_id, $software_id, $qualifier, $qvalue) = @_;
    my ($sql, $sth, $rc, $ok);

    $qvalue =~ s/\'/\'\'/g;
    $sql = "update metabolite_info_detail set $qualifier = \'$qvalue\' where metabolite_info_detail_id = $metabolite_info_detail_id and software_id = $software_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    
    $sth->finish;
    return $rc;
}

###########################################################################
# createQualifierId($dbh, $mid_kwid, $mid_id, $qualifier, $qualifier_value)
#
# parameters in: $dbh -- the database handle,
#                $mid_kwid -- kw_table_id for metabolite_info_detail 
#                $mid_id -- metabolite_info_detail_id
#                $qualifier -- 
#                $qualifier_value -- qualifier value
# parameter out: $metabolite_info_detail_id -- the new id
#
# X. Jing Li, 9/25/2006
###########################################################################
sub createQualifierId {
    my ($dbh, $mid_kwid, $mid_id, $qualifier, $qualifier_value) = @_;
    my ($sql, $sth, $rc, $qualifier_id);

    $sql = "select nextval('qualifier_id_seq') ";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $qualifier_id = $sth->fetchrow();

    $qualifier =~ s/\'/\'\'/g;
    $qualifier_value =~ s/\'/\'\'/g;
    $sql = "insert into qualifier (qualifier_id, kw_table_id, record_id, qualifier_name, qualifier_value) values ($qualifier_id, $mid_kwid, $mid_id, \'$qualifier\', \'$qualifier_value\')";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $sth->finish;
    return $qualifier_id;
}
